# Generated by Django 3.1.10 on 2021-06-22 16:27

import hashlib

from django.db import connection, migrations, models


def gen_raw_md5(apps, schema_editor):
    """Custom update of _raw_md5 hash."""
    RawNotificationModel = apps.get_model("nautobot_circuit_maintenance", "RawNotification")

    for raw_notification in RawNotificationModel.objects.all():
        raw_notification._raw_md5 = hashlib.md5(raw_notification.raw.encode("utf-8")).hexdigest()  # noqa: S324
        raw_notification.save(update_fields=["_raw_md5"])


class Migration(migrations.Migration):
    dependencies = [
        ("nautobot_circuit_maintenance", "0003_improve_rawnotification"),
    ]

    if connection.vendor == "postgresql":
        operations = [
            migrations.AddField(
                model_name="rawnotification",
                name="_raw_md5",
                field=models.TextField(null=True),
            ),
            # TODO: Move data migration into a separate file and add a reverse migration
            migrations.RunPython(code=gen_raw_md5, reverse_code=migrations.RunPython.noop),
            migrations.AlterField(
                model_name="rawnotification",
                name="_raw_md5",
                field=models.TextField(unique=True),
            ),
            migrations.AlterField(
                model_name="rawnotification",
                name="raw",
                field=models.TextField(),
            ),
        ]
    elif connection.vendor == "mysql":
        operations = [
            migrations.AddField(
                model_name="rawnotification",
                name="_raw_md5",
                field=models.CharField(null=True, max_length=512),
            ),
            # TODO: Move data migration into a separate file and add a reverse migration
            migrations.RunPython(code=gen_raw_md5, reverse_code=migrations.RunPython.noop),
            migrations.AlterField(
                model_name="rawnotification",
                name="_raw_md5",
                field=models.CharField(unique=True, max_length=512),
            ),
            migrations.AlterField(
                model_name="rawnotification",
                name="raw",
                field=models.CharField(max_length=512),
            ),
        ]
